% init.m
%
% This script initializes various things: Productivity process, EPL scheme, value functions of older non-employed
% workers workers, initial guesses for value functions, initial guesses for wage functions, etc.


%%   Set parameters.

r    = 0.01;                 T     = 120;                  % Discount factor / Maximum tenure
beta = 0.5;                  psi   = 0.5;                  % Bargaining power / Matching function elasticity
A    = 0.40;                 delta = 0.005;                % Matching efficiency / Exogenous separation rate
if exist('chi','var') == 0                                 % Check compstat.m --> Do not overwrite if already specified
  chi = 1/40;                gamma = 1/120;                % Retirement probability / Probability of becoming older
  upsil = 1.0;                                             % One minus red-tape component  
end
  

%%   Miscellaneous.

z = linspace(0,1,nz)';                                     % Create grid over productivity levels (equidistant)
P = zeros(nz,nz);                                          % Initialize

if (exist('wx0','var')) || (exist('z0disp','var'))         % Use non-equidistant distribution of grid points for z
  z = [ linspace(0,0.08,5)' ; linspace(0.10,0.35,81)' ; linspace(0.36,1,35)' ];
  z0idx = sum(z <= z0);
end

for j = 1 : nz
  P(j,1) = P(j,1) + normcdf(0.5*(z(1)+z(2))-z(j),0,stdz);
  P(j,2:nz-1) = P(j,2:nz-1) + normcdf(0.5*(z(2:nz-1)+z(3:nz  ))-z(j),0,stdz)' ...
                            - normcdf(0.5*(z(1:nz-2)+z(2:nz-1))-z(j),0,stdz)';
  P(j,end) = P(j,end) + 1 - normcdf(0.5*(z(end)+z(end-1))-z(j),0,stdz);
end

kappa = 0.01*kappa;                                        % Convert percentage value (payroll tax)
N     = length(P);                                         % Number of productivity levels

io1 = 10;   iy1 = 5;   iy2 = 2;   xx = 0.60;               % Number of iterations / Dampening parameter


%%   Specify EPL scheme.

phi = zeros(T,1);   coef1 = 8/365;   coef2 = 45/365;       % Coefficients for benchmark EPL scheme
for i1 = 2 : 9                                             % Temporary jobs (first 8 quarters)
   phi(i1) = coef1*wavg0*(i1-1);
end
for i1 = 10 : 114                                          % Permanent jobs (9 quarters or more)
   phi(i1) = coef2*wavg0*(i1-1);
end
phi(115:T) = (wavg0*4)*3.5;                                % Max severance payment: 42 monthly wages (3.5 yearly)

if uniEPL == 1                                             % Unified EPL scheme
  phi = max( 0 , (rho_u/365)*wavg0 * ([ 0 : 1 : T-1 ]' - tau_u/3) );
end

philag = phi;   philag(1:end-1) = phi(2:end);   philag(end) = philag(end-1);


%%   Miscellaneous.

if exist('rho_u','var') == 0,   fprintf('       ');   end
annfaco  = (r/(1+r)) * (1/(1-(1+r)^inv(-chi)));            % Annuity factor: Older workers
btildeoA = bo + upsil*annfaco*phi;                         % Non-employment income for older workers
UoA      = -(1./btildeoA-1)/(1-(1/(1+r))*(1-chi));         % Non-employed older workers: Laid off when old
V0       = -0.9*(1./wavg0-1)/(1-(1/(1+r)));                % Initial guess for value functions

U   = V0*ones(T,1);                W     = V0;                       J   = 1;
w   = wavg0;                       theta = 1;
Wo  = repmat(UoA,1,N);             Jo    = -1*ones(T,N);             wo  = wavg0*ones(T,N);
wo1 = wo;                          woF   = wo;                       woW = w;
W   = W*ones(T,N);                 J     = J*ones(T,N);              w   = w*ones(T,N);
w1  = w;                           wF    = w;                        wW  = w;
aux = min(2:T+1,T);

if exist('wx0','var')                                      % For model with rigid wage: Initialize
  Wx  = W;  Jx  = J;  Wox = Wo;  Jox = Jo;  critwx = 100;  itwx = 1;
end


%%   Model with dispersion in initial productivity.

if (exist('z0disp','var')) 
  z0centeridx       = sum(z <= z0);                        % Mean
  stdz0             = stdz;                                % Standard deviation
  z0distrib         = zeros(1,nz);                         % Initialize
  z0distrib(1)      = normcdf(0.5*(z(1)+z(2))-z(z0centeridx),0,stdz0);
  z0distrib(2:nz-1) = normcdf(0.5*(z(2:nz-1)+z(3:nz  ))-z(z0centeridx),0,stdz0)' ...
                    - normcdf(0.5*(z(1:nz-2)+z(2:nz-1))-z(z0centeridx),0,stdz0)';
  z0distrib(end)    = 1 - normcdf(0.5*(z(end)+z(end-1))-z(z0centeridx),0,stdz0);
  
  W0  = zeros(T,N);        J0     = zeros(T,N);            % Value functions for initial matches
  w0  = zeros(T,N);        split0 = zeros(T,N);            % Wages and separation rules for initial matches
  wW0 = repmat(0.3,T,N);   wF0    = repmat(0.3,T,N);       % Wages for initial matches
  
  xx_pjob = 0.01;                                          % Damping parameter for updating (pjob)
  xx      = 0.10;                                          % Damping parameter for updating (wages w)
  xx0     = 0.10;                                          % Damping parameter for updating (wages w0)
end


%%   Model with human capital accumulation.

if (exist('Hprem','var'))
  WH = W;   JH = J;   wH = w;   WoH = Wo;   JoH = Jo;   woH = wo;   wo1H = wo1;
  woFH = woF;   woWH = woW;    w1H = w1;   wFH = wF;   wWH = wW;
end