!---------------------------------------------------------------------------------------------------------------------------------------------------
!	module solve_model: solve the model for a given set of parameters
!---------------------------------------------------------------------------------------------------------------------------------------------------

module solve_model

	use globals
	use misc_functions
	use wage_function_old
	use wage_function_young
	use distribution
		
	implicit none
	
contains
	

!	subroutine to compute the model equilibrium

subroutine solve_transition_path()

	double precision :: skip, kappa_t1
	double precision :: ERR_kappa, NEW_kappa(NP), upd_kappa = 0.15d+0
	integer :: NUM_kappa
	
	print *, ' '
	print *, 'Iterating on the time path of the payroll tax'
	print *, ' '
	
	!	begin iteration
	
	NUM_kappa = 0
	ERR_kappa = 1.0d+2
	
	do while (ERR_kappa .gt. TOL_kappa)
				
		!	solve the model backwards
		
		call compute_model_backwards()
		
		!	compute the time-path for kappa
				
		do ip = 1, NP
				
			!	equilibrium tax rate
			
			call compute_avg_wage()			
			NEW_kappa(ip) = (By*sum(LU(:,:,1))+Bo*sum(LU(:,:,2:3)))/(avg_w*sum(LE))
		
			!	current distribution
			
			call update_distribution()
		
		end do
			
		!	check convergence
		
		ERR_kappa = maxval(dabs(NEW_kappa - kappa))
			
		!	update kappa
		
		kappa(1:NP-1) = (1.0d+0-upd_kappa)*kappa(1:NP-1) + upd_kappa*NEW_kappa(1:NP-1)
		
		!	stop if too long
		
		if ( NUM_kappa .eq. MAX_kappa ) exit
		
		!	move on
		
		NUM_kappa = NUM_kappa + 1
		
	end do
		
	if (ERR_kappa .gt. TOL_kappa) then
		print *, 'force solve_transition_path() to exit'
		print *, 'last error term was ', ERR_kappa
		print *, ' '
	end if
	
	!	display information	
	
	open(1, file = 't1/eq_tuple.txt', status = 'old')
	read(1, *) skip
	read(1, *) kappa_t1
	close(1)
			
	print *, ' '
	print *, 'terminal period, comparison'
	write(*, '(X,A,X,F7.4)') 'path: ', 1.0d+2*kappa(NP-1)
	write(*, '(X,A,X,F7.4)') 'exact:', 1.0d+2*kappa_t1
	print *, ' '
	print *, 'number of iteration', NUM_kappa-1
	
end subroutine


!	subroutine to solve backwards from t1

subroutine compute_model_backwards()

	!	initialize various functions
	
	call initialize_various_functions()

	!	first period
	
	ip = NP
	
	call retrieve_policy_functions()
	
	!	move on
	
	do ip = NP-1, 1, -1

		!	retrieve layoff payments
		
		call initialize_layoff_payments()
	
		!	obtain wages for old workers
				
		call solve_wage_old_workers()
		
		!	equilibrium tightness
		
		theta(ip) = theta_free_entry()
		
		!	job-finding rate
		
		jfind = min(A*theta(ip)**(1.0d+0-psi),theta(ip),1.0d+0)
		
		!	expected duration of unemployment spell
		
		Edur = (1.0d+0-gamma)*(jfind + (1.0d+0-jfind)*(EdurP+1.0d+0)) + gamma/khi
		
		!	value of young unemployed
		
		call value_unemployed_young()
		
		!	obtain wages for young workers
		
		call solve_wage_young_workers()
		
		!	retrieve policy functions
		
		call retrieve_policy_functions()
		
		!	move on
		
		Wn(ip)= Wy(1,iz0,NC)
		UyP   = Uy
		EdurP = Edur
		
	end do
	
end subroutine


!	compute average wage in the economy

subroutine compute_avg_wage()

	double precision :: res
	
	avg_w = 0.0d+0
	do it = 1, NT
	do iz = 1, NZ
	do ic = 1, NC
		avg_w = avg_w + wwy(ip,it,iz,ic)*LE(it,iz,ic,1) + wwo(ip,it,iz,ic)*LE(it,iz,ic,2)
	end do
	end do
	end do
	avg_w = avg_w/sum(LE)
	
end subroutine


!	obtain theta consistent with free-entry

double precision function theta_free_entry()
	
	double precision :: res

	res = rho*Jy(1,iz0,NC)
	res = k/res
	theta_free_entry = min(1.0d+0/res, (A/res)**(1.0d+0/psi))

end function


!	subroutine to obtain policy functions for match continuation decisions

subroutine retrieve_policy_functions()

	double precision :: resa_U
	
	sepy(ip, :, :) = NZ+1
	sepo(ip, :, :) = NZ+1

	do it = NT, 1, -1
	do iz = NZ, 1, -1
	do ic = NC, 1, -1
	
		resa_U = Uy(it,ic) + (1.0d+0-young_sp)*(utils(By+nu*phi(it,ic))-utils(By))
		if ( min(Jy(it,iz,ic) + phi(it,ic), Wy(it,iz,ic)- resa_U) .le. 0.0d+0 ) sepy(ip,it,ic) = iz
		if ( min(Jo(it,iz,ic) + phi(it,ic), Wo(it,iz,ic)-Uo(it,ic)) .le. 0.0d+0 ) sepo(ip,it,ic) = iz
		
	end do
	end do
	end do
	
end subroutine

end module