!---------------------------------------------------------------------------------------------------------------------------------------------------
!	module solve_model: solve the model for a given set of parameters
!---------------------------------------------------------------------------------------------------------------------------------------------------

module solve_model

	use globals
	use misc_functions
	use wage_function_old
	use wage_function_young
	use distribution
		
	implicit none
	
contains
	

!	subroutine to compute the model equilibrium

subroutine compute_model_equilibrium()

	double precision :: ERR_theta, ERR_budg, ERR_avgw
	double precision :: upd_theta = 0.25d+0, upd_kappa = 0.50d+0, upd_avgw = 0.25d+0, lag_avgw
	integer :: NUM_theta, NUM_budg
	
	!	begin iteration on the average wage
	
	ERR_avgw  = 1.0d+2
	
	do while (ERR_avgw .gt. TOL_avgw)
	
		!	store lagged average wage
		
		lag_avgw = avg_w
		
		!	set the layoff payment function
		
		call initialize_layoff_payments()
		
		!	control error convergence
		
		ERR_cvg = 0
		
		!	begin iteration on the budget constraint
		
		NUM_budg  = 1
		ERR_budg  = 1.0d+2
		
		do while (ERR_budg .gt. TOL_budg)
			
			!	begin iteration on equilibrium tightness
			
			NUM_theta  = 1
			ERR_theta  = 1.0d+2
			
			do while (ERR_theta .gt. TOL_theta)
			
				!	job-finding rate
				
				jfind = min(A*theta**(1.0d+0-psi),theta,1.0d+0)
			
				!	obtain wages
				
				call solve_wage_old_workers()
				call solve_wage_young_workers()
				
				!	check convergence
				
				ERR_theta = abs(theta_free_entry() - theta)
				
				!	update theta
				
				theta = (1.0d+0-upd_theta)*theta + upd_theta*theta_free_entry()
				
				!	move on
				
				NUM_theta = NUM_theta + 1
				
				!	stop if too long
		
				if (NUM_theta .gt. MAX_theta) exit
				
			end do
			
			!	error message
			
			if (ERR_theta .gt. TOL_theta) then
				print *, 'force solve_equilibrium_tightness() to exit'
				print *, 'last error term was ', ERR_theta
				print *, ' '
			end if
			
			!	compute time-invariant distribution
		
			call separation_rules()
			call invariant()
			
			!	obtain average wage
			
			avg_w = compute_avg_wage()
	
			!	check convergence
			
			ERR_budg = abs(kappa_budget_eq() - kappa)
			
			!	update kappa
			
			kappa = (1.0d+0-upd_kappa)*kappa + upd_kappa*kappa_budget_eq()
			
			!	move on
			
			NUM_budg = NUM_budg + 1
			!print *, NUM_budg, kappa, sum(LU), avg_w
			
			!	stop if too long
	
			if (NUM_budg .gt. MAX_budg) exit
			
			!	display results
			
			call display_statistics()
						
		end do
		
		print *, ' '
		print *, ' '
		if ( ERR_cvg .eq. 1 ) print *, 'some bellman equations failed to converge'
		
		!	check convergence
		
		ERR_avgw = abs(lag_avgw - avg_w)
		
		!	update kappa
		
		avg_w = (1.0d+0-upd_avgw)*avg_w + upd_avgw*lag_avgw
		
	end do
	

end subroutine


!	subroutine to compute various statistics

subroutine display_statistics()

	double precision :: avg_w_y, avg_z_y, avg_w_o, avg_z_o, avg_z, norm
	double precision :: sep_y, sep_o, sep_temp, sep_perm, sep_tot, sep(NZ), u_dur(8)
	integer :: ix
	
	print *, ' '
	
	!	average wages
	
	avg_w_y = 0.0d+0
	avg_w_o = 0.0d+0
	do it = 1, NT
	do iz = 1, NZ
		if (it .eq. 1) avg_w_y = avg_w_y + acc(it,iz)*wwy0(it,iz)*(1.0d+0-gamma)*alpha*jfind*P0(iz)*LU(it,1)
		if (it .gt. 1) avg_w_y = avg_w_y + wwy(it,iz)*LE(it,iz,1)
		avg_w_o = avg_w_o + wwo(it,iz)*LE(it,iz,2)
	end do
	end do
	avg_w_y = avg_w_y/sum(LE(:,:,1))
	avg_w_o = avg_w_o/sum(LE(:,:,2))
	
	print *, 'Average wage in firing cost function', phi(NT)/14.0d+0
	print *, 'Average wage: ', 'young:', avg_w_y, 'old: ', avg_w_o
	print *, 'all:', avg_w
	print *, 'By/avg_w (%)', 1.0d+2*By/avg_w_y
	print *, 'Bo/avg_w (%)', 1.0d+2*Bo/avg_w_o
	print *, ' R/avg_w (%)', 1.0d+2*inv_utils((1.0d+0-rho)*R-leis)/avg_w
	
	
	!	productivity levels
	
	avg_z_y = dot_product(zGrid, sum(LE(:,:,1), dim = 1))/sum(LE(:,:,1))
	avg_z_o = dot_product(zGrid, sum(LE(:,:,2), dim = 1))/sum(LE(:,:,2))
	avg_z   = dot_product(zGrid, sum(LE(:,:,1) + LE(:,:,2), dim = 1))/sum(LE)
	
	print *, 'Average prod: ', 'young:', avg_z_y, 'old: ', avg_z_o
	print *, 'all:', avg_z 
	
	
	!	tenure distribution
	
	print *, ' '
	print *, 'Tenure less than 2 years : ', 1.0d+2*sum(LE(1:7,:,:))/sum(LE)
	print *, 'Tenure from 2 to 5 years : ', 1.0d+2*sum(LE(8:19,:,:))/sum(LE)
	print *, 'Tenure from 5 to 10 years: ', 1.0d+2*sum(LE(20:39,:,:))/sum(LE)
	print *, 'Tenure more than 10 years: ', 1.0d+2*sum(LE(40:NT,:,:))/sum(LE)
	
	
	!	separation rates

	!	young workers
	
	sep_y = 0.0d+0
	do it = 2, NT-1
	
		sep	= (1.0d+0-delta)*(1.0d+0-gamma)*matmul(transpose(P), LE(it-1,:,1))
		if (sepy(it) .le. NZ) sep_y = sep_y + sum(sep(sepy(it):NZ))
		sep = (1.0d+0-delta)*gamma*matmul(transpose(P), LE(it-1,:,1))
		if (sepo(it) .le. NZ) sep_y = sep_y + sum(sep(sepo(it):NZ))
	
	end do
	sep = (1.0d+0-delta)*(1.0d+0-gamma)*matmul(transpose(P), LE(NT-1,:,1) + LE(NT,:,1))
	if (sepy(NT) .le. NZ) sep_y = sep_y + sum(sep(sepy(NT):NZ))
	sep = (1.0d+0-delta)*gamma*matmul(transpose(P), LE(NT-1,:,1) + LE(NT,:,1))
	if (sepo(NT) .le. NZ) sep_y = sep_y + sum(sep(sepo(NT):NZ))
	sep_y = sep_y + delta*sum(LE(:,:,1))
	
	!	old workers
	
	sep_o = 0.0d+0
	do it = 2, NT-1
		
		sep	= (1.0d+0-delta)*(1.0d+0-khi)*matmul(transpose(P), LE(it-1,:,2))
		if (sepo(it) .le. NZ) sep_o = sep_o + sum(sep(sepo(it):NZ))
		
	end do
	sep = (1.0d+0-delta)*(1.0d+0-khi)*matmul(transpose(P), LE(NT-1,:,2) + LE(NT,:,2))
	if (sepo(NT) .le. NZ) sep_o = sep_o + sum(sep(sepo(NT):NZ))
	sep_o = sep_o + khi*sum(LE(:,:,2))
	sep_o = sep_o + (1.0d+0-khi)*delta*sum(LE(:,:,2))
	
	!	first 2 years
	
	sep_temp = 0.0d+0
	do it = 2, 9
		
		sep = (1.0d+0-delta)*(1.0d+0-gamma)*matmul(transpose(P), LE(it-1,:,1))
		if (sepy(it) .le. NZ) sep_temp = sep_temp + sum(sep(sepy(it):NZ))		
		sep = (1.0d+0-delta)*(gamma*matmul(transpose(P), LE(it-1,:,1)) + (1.0d+0-khi)*matmul(transpose(P), LE(it-1,:,2)))
		if (sepo(it) .le. NZ) sep_temp = sep_temp + sum(sep(sepo(it):NZ))
		
	end do
	sep_temp = sep_temp + khi*sum(LE(1:8,:,2))
	sep_temp = sep_temp + delta*(sum(LE(1:8,:,1)) + (1.0d+0-khi)*sum(LE(1:8,:,2)))
	
	!	remaining separations
	
	sep_tot  = sep_y + sep_o
	sep_perm = sep_tot-sep_temp
	
	!	scale
	
	sep_y	 = sep_y/sum(LE(:,:,1))
	sep_o	 = sep_o/sum(LE(:,:,2))
	sep_temp = sep_temp/sum(LE(1:8,:,1)+LE(1:8,:,2))
	sep_perm = sep_perm/sum(LE(9:NT,:,1)+LE(9:NT,:,2))
	sep_tot  = sep_tot/sum(LE)
	
	!	unemployment duration
	
	u_dur = 0.0d+0
	
	do ix = 1, 8
	
		do it = 1, NT 
			u_dur(ix) = u_dur(ix) + LU(it,1)*((1.0d+0-(1.0d+0-gamma)*jfind*(alpha*sum(acc(it,:)*P0)+1.0e+0-alpha))**dble(ix))
		end do
		
	end do
	
	u_dur = u_dur/sum(LU(:,1))
	
	print *, '% unemployed for 6 to 12 months', 1.0d+2*(u_dur(2)-u_dur(4))
	print *, '% unemployed for 1 to 2 years', 1.0d+2*(u_dur(4)-u_dur(8))
	print *, '% unemployed for 2 years and more', 1.0d+2*u_dur(8)
	print *, ' '
	print *, ' '
	print *, 'Separation rate ', 'young:', 1.0d+2*sep_y, 'old: ', 1.0d+2*sep_o
	print *, 'Separation rate ', 'temp :', 1.0d+2*sep_temp, 'perm:', 1.0d+2*sep_perm
	print *, 'Separation rate (total)', 1.0d+2*sep_tot
	print *, 'Quit rate (total)', 1.0d+2*delta*(sum(LE(:,:,1))+(1.0d+0-khi)*sum(LE(:,:,2)))/sum(LE)
	print *, 'Quit rate (/total)', 1.0d+2*delta*((sum(LE(:,:,1))+(1.0d+0-khi)*sum(LE(:,:,2)))/sum(LE))/sep_tot
	print *, 'Job-finding rate', 1.0d+2*jfind
	print *, 'Vacancy cost    ', k
	print *, ' '
	
	!	nonemployment
	
	norm = sum(LU(:,1))+sum(LE(:,:,1))
	print *, 'Nonemployment rate ', 'young:', 1.0d+2*sum(LU(:,1))/norm, 'old: ', 1.0d+2*sum(LU(:,2:3))/(1.0d+0-norm)
	print *, 'Nonemployment rate ', 1.0d+2*sum(LU)
	print *, ' '
	
	print *, 'Equilibrium tightness ', theta
	print *, 'Equilibrium tax rate', 1.0d+2*kappa
	print *, ' '
	
	print *, 'Lifetime utility, newborn', Uy(1)
	print *, ' '
	print *, ' '
	
end subroutine


!	obtain kappa consistent with balanced budget constraint

double precision function kappa_budget_eq()

	kappa_budget_eq = (By*sum(LU(:,1))+Bo*sum(LU(:,2:3)))/(avg_w*sum(LE))
	
end function


!	compute average wage in the economy

double precision function compute_avg_wage()

	double precision :: res
	
	res = 0.0d+0
	
	do it = 1, NT
	do iz = 1, NZ
		if (it .eq. 1) res = res + acc(it,iz)*wwy0(it,iz)*(1.0d+0-gamma)*alpha*jfind*P0(iz)*LU(it,1)
		if (it .gt. 1) res = res + wwy(it,iz)*LE(it,iz,1)
		res = res + wwo(it,iz)*LE(it,iz,2)
	end do
	end do
	compute_avg_wage = res/sum(LE)
	
end function


!	obtain theta consistent with free-entry

double precision function theta_free_entry()

	double precision :: res, DPV
	
	DPV = 0.0d+0
	do it = 1, NT
		DPV = DPV + alpha*dot_product(max(acc(it,:)*Jy0(it,:),0.0d+0),P0)*LU(it,1)/sum(LU(:,1))
	end do
	DPV = DPV + (1.0d+0-alpha)*Jy(1,iz0)
	
	if (norm_theta) then
	
		k = A*rho*DPV
		theta_free_entry = 1.0d+0
		
	else
	
		res = rho*DPV
		res = k/res
		theta_free_entry = min(1.0d+0/res, (A/res)**(1.0d+0/psi))

	end if
		
end function

end module