!---------------------------------------------------------------------------------------------------------------------------------------------------
!	module wage_function_old: obtain wage functions for older workers
!---------------------------------------------------------------------------------------------------------------------------------------------------

module wage_function_old

	use globals
	use misc_functions
	
	implicit none
	
	double precision :: contW, contJ
	
contains


!	subroutine to solve for wages

subroutine solve_wage_old_workers()


	!	solve terminal problem

	call solve_terminal_problem()
	
	!	solve backwards from T-1
	
	do it = NT-1, 1, -1
	do iz = 1, NZ

		!	continuation values here
			
		call continuation_old_workers()

		!	reservation wages
				
		wmin = inv_utils(Uo(it) - contW)
		wmax = (phi(it) + zGrid(iz) + contJ)/(1.0d+0+kappa)
		
		!	solve for the equilibrium wage
		
		if ( wmin .le. wmax ) then
		
			wwo(it,iz) = sqrt((zGrid(iz) + contJ + phi(it))/((1.0d+0 + contW - Uo(it))*(1.0d+0+kappa)))
			!	wwo(it,iz) = solve_nash_foc()
			
		else
		
			wwo(it,iz) = (wmin+wmax)/2.0d+0
			
		end if
		
		!	fill the corresponding value functions
		
		Wo(it,iz) = utils(wwo(it,iz)) + contW
		Jo(it,iz) = zGrid(iz)-(1.0d+0+kappa)*wwo(it,iz) + contJ
	
	end do
	end do
		
end subroutine


!	continuation values

subroutine continuation_old_workers()

	!	worker's side
	
	contW = rho*((1.0d+0-khi)*(delta*(Uo(1) + delta_sp*(Uo(min(NT,it+1))-Uo(1))) &
			+ (1.0d+0-delta)*dot_product(P(iz,:),max(Wo(min(NT,it+1),:),Uo(min(NT,it+1))))) + khi*R)

	!	firm's side
	
	contJ = rho*(1.0d+0-khi)*(-delta*phi(min(NT,it+1))*delta_sp &
			+ (1.0d+0-delta)*dot_product(P(iz,:),max(Jo(min(NT,it+1),:),-phi(min(NT,it+1)))))
		
end subroutine
		

!	solve problem at time T

subroutine solve_terminal_problem()

	double precision :: NewJ(NZ), NewW(NZ), Newww(NZ), Wres(NZ), Jres(NZ)
	double precision :: ERR_bell
	integer :: NUM_bell
	
	!	set terminal period
	
	it = NT	
	
	!	begin iterations
	
	NUM_bell = 1
	ERR_bell = 1.0d+2
	
	do while (ERR_bell .gt. TOL_bell)

		do iz = 1, NZ

			!	continuation values
			
			call continuation_old_workers()
		
			!	value functions
			
			NewW(iz) = utils(wwo(it,iz)) + contW
			NewJ(iz) = zGrid(iz) - (1.0d+0+kappa)*wwo(it,iz) + contJ
				
			!	reservation wage for workers
			
			Wres = max(Wo(NT,:),Uo(NT))
			Wres(iz) = Uo(NT)
			wmin = inv_utils(Uo(NT) - rho*((1.0d+0-khi)*(delta*(Uo(1) + delta_sp*(Uo(NT)-Uo(1))) &
												+ (1.0d+0-delta)*dot_product(P(iz,:),Wres)) + khi*R))
			
			!	reservation wage for firms
			
			Jres = max(Jo(NT,:),-phi(NT))
			Jres(iz) = -phi(NT)
			wmax = (phi(NT) + zGrid(iz) + rho*(1.0d+0-khi)*(-delta*phi(NT)*delta_sp &
												+ (1.0d+0-delta)*dot_product(P(iz,:),Jres)))/(1.0d+0+kappa)
			
			!	update wage function

			if ( wmin .le. wmax ) then
				
				Newww(iz) = sqrt((zGrid(iz) + contJ + phi(NT))/((1.0d+0 + contW - Uo(NT))*(1.0d+0+kappa)))
				!	Newww(iz) = solve_nash_foc()
				
			else
							
				Newww(iz) = (wmin+wmax)/2.0d+0
				
			end if

		end do
		
		!	check convergence
		
		ERR_bell = max(maxval(dabs(Wo(it,:) - NewW)), maxval(dabs(Jo(it,:) - NewJ)), maxval(dabs(wwo(it,:) - Newww)))

		!	stop if too long
		
		if (NUM_bell .gt. MAX_bell) exit
		
		!	update value functions
		
		Wo(it,:)  = NewW
		Jo(it,:)  = NewJ
		wwo(it,:) = Newww
		NUM_bell = NUM_bell + 1
		
	end do

	!	print *, 'value functions for older workers have converged'
	
	if (ERR_bell .gt. TOL_bell) then
		print *, 'force solve_bellman() to exit (older workers)'
		print *, 'last error term was ', ERR_bell
		print *, ' '
		ERR_cvg = 1
	else
		ERR_cvg = 0
	end if
	
end subroutine


!	solve Nash problem using FOC (bisection method)

double precision function solve_nash_foc()

	double precision :: x, xa, xb, ya, yb, y
	logical :: ifstop
	
	!	minimum starting point
	xa = wmin
	ya = nash_foc(xa)
	
	!	maximum starting point
	xb = wmax
	yb = nash_foc(xb)
			
	ifstop = .true.
	do while (ifstop)
		
		x = (xa+xb)/2.0d+0
		y = nash_foc(x)
		if ( y*ya .lt. 0.0d+0 ) then
			xb = x
			yb = y
		else
			xa = x
			ya = y
		end if
		
		if (abs(xb-xa) .lt. TOL_nash) ifstop = .false.
		
	end do
	solve_nash_foc = (xa+xb)/2.0d+0
	
end function


!	write down Nash FOC

double precision function nash_foc(ww)
	
	double precision, intent(in) :: ww
	double precision :: valW, valJ
	
	!	worker's side
	
	valW = utils(ww) + contW

	!	firm's side
	
	valJ = zGrid(iz) - (1.0d+0+kappa)*ww  + contJ
		
	!	difference
	
	nash_foc = beta*(valJ + phi(it))/(1.0d+0+kappa) - (1.0d+0-beta)*(valW-Uo(it))/umarg(ww)
	
end function


end module