!---------------------------------------------------------------------------------------------------------------------------------------------------
!	module solve_model: solve the model for a given set of parameters
!---------------------------------------------------------------------------------------------------------------------------------------------------

module solve_model

	use globals
	use misc_functions
	use wage_function_old
	use wage_function_young
	use distribution
		
	implicit none
	
contains
	

!	subroutine to compute the model equilibrium

subroutine compute_model_equilibrium()

	double precision :: ERR_theta, ERR_budg, ERR_avgw
	double precision :: upd_theta = 0.25d+0, upd_kappa = 0.25d+0, upd_avgw = 0.10d+0, lag_avgw
	integer :: NUM_theta, NUM_budg
	
	!	begin iteration on the average wage
	
	ERR_avgw  = 1.0d+2
	
	do while (ERR_avgw .gt. TOL_avgw)
	
		!	store lagged average wage
		
		lag_avgw = avg_w
		
		!	set the layoff payment function
		
		call initialize_layoff_payments()
		
		!	control error convergence
		
		ERR_cvg = 0
		
		!	begin iteration on the budget constraint
		
		NUM_budg  = 1
		ERR_budg  = 1.0d+2
		
		do while (ERR_budg .gt. TOL_budg)
			
			!	begin iteration on equilibrium tightness
			
			NUM_theta  = 1
			ERR_theta  = 1.0d+2
			
			do while (ERR_theta .gt. TOL_theta)
			
				!	job-finding rate
				
				jfind = min(A*theta**(1.0d+0-psi),theta,1.0d+0)
			
				!	obtain wages
				
				call solve_wage_old_workers()
				call solve_wage_young_workers()
				
				!	check convergence
				
				ERR_theta = abs(theta_free_entry() - theta)
				
				!	update theta
				
				theta = (1.0d+0-upd_theta)*theta + upd_theta*theta_free_entry()
				
				!	move on
				
				NUM_theta = NUM_theta + 1
				
				!	stop if too long
		
				if (NUM_theta .gt. MAX_theta) exit
				
			end do
			
			!	error message
			
			if (ERR_theta .gt. TOL_theta) then
				print *, 'force solve_equilibrium_tightness() to exit'
				print *, 'last error term was ', ERR_theta
				print *, ' '
			end if
			
			!	compute time-invariant distribution
			
			call separation_rules()
			call invariant()
			
			!	obtain average wage
			
			avg_w = compute_avg_wage()
			
			!	check convergence
			
			ERR_budg = abs(kappa_budget_eq() - kappa)
			
			!	update kappa
			
			kappa = (1.0d+0-upd_kappa)*kappa + upd_kappa*kappa_budget_eq()
			
			!	move on
			
			NUM_budg = NUM_budg + 1
			
			!	stop if too long
	
			if (NUM_budg .gt. MAX_budg) exit
			
			!	display results
			
			!call display_statistics()
						
		end do
		
		print *, ' '
		print *, ' '
		if ( ERR_cvg .eq. 1 ) print *, 'some bellman equations failed to converge'
		
		!	check convergence
		
		ERR_avgw = abs(lag_avgw - avg_w)
		
		!	update kappa
		
		avg_w = (1.0d+0-upd_avgw)*avg_w + upd_avgw*lag_avgw
		
	end do
	

end subroutine


!	subroutine to compute various statistics

subroutine display_statistics()

	double precision :: avg_w_y, avg_z_y, avg_w_o, avg_z_o, avg_z, norm
	double precision :: sep_y, sep_o, sep_temp, sep_perm, sep_tot, sep(NZ)
	
	print *, ' '
	
	!	average wages
	
	avg_w_y = 0.0d+0
	avg_w_o = 0.0d+0
	do it = 1, NT
	do iz = 1, NZ
		avg_w_y = avg_w_y + wwy_0(it,iz)*LE_0(it,iz,1) + wwy_1(it,iz)*LE_1(it,iz,1)
		avg_w_o = avg_w_o + wwo_0(it,iz)*LE_0(it,iz,2) + wwo_1(it,iz)*LE_1(it,iz,2)
	end do
	end do
	avg_w_y = avg_w_y/sum(LE_0(:,:,1)+LE_1(:,:,1))
	avg_w_o = avg_w_o/sum(LE_0(:,:,2)+LE_1(:,:,2))
	
	print *, 'Average wage in firing cost function', phi(NT)/14.0d+0
	print *, 'Average wage: ', 'young:', avg_w_y, 'old: ', avg_w_o
	print *, 'all:', avg_w
	print *, '% workers with HC: ', 1.0d+2*sum(LE_1)/(1.0d+0-sum(LU))
	print *, '% workers with HC: ','young:',1.0d+2*sum(LE_1(:,:,1))/sum(LE_0(:,:,1)+LE_1(:,:,1)),&
							  	   'old: ',1.0d+2*sum(LE_1(:,:,2))/sum(LE_0(:,:,2)+LE_1(:,:,2))
							  	   
	print *, ' '
	print *, 'HC at tenure less than 2 years : ', 1.0d+2*sum(LE_1(1:7,:,:))/sum(LE_0(1:7,:,:)+LE_1(1:7,:,:))
	print *, 'HC at tenure from 2 to 5 years : ', 1.0d+2*sum(LE_1(8:19,:,:))/sum(LE_0(8:19,:,:)+LE_1(8:19,:,:))
	print *, 'HC at tenure from 5 to 10 years: ', 1.0d+2*sum(LE_1(20:39,:,:))/sum(LE_0(20:39,:,:)+LE_1(20:39,:,:))
	print *, 'HC at tenure more than 10 years: ', 1.0d+2*sum(LE_1(40:NT,:,:))/sum(LE_0(40:NT,:,:)+LE_1(40:NT,:,:))
								  	   
	
	print *, 'By/avg_w (%)', 1.0d+2*By/avg_w_y
	print *, 'Bo/avg_w (%)', 1.0d+2*Bo/avg_w_o
	print *, ' R/avg_w (%)', 1.0d+2*inv_utils((1.0d+0-rho)*R-leis)/avg_w
	
	
	!	productivity levels
	
	avg_z_y = (dot_product(zGrid, sum(LE_0(:,:,1), dim = 1)) &
				+dot_product(zGrid*(1.0d+0+H), sum(LE_1(:,:,1), dim = 1)))/sum(LE_0(:,:,1)+LE_1(:,:,1))
	avg_z_o = (dot_product(zGrid, sum(LE_0(:,:,2), dim = 1)) &
				+dot_product(zGrid*(1.0d+0+H), sum(LE_1(:,:,2), dim = 1)))/sum(LE_0(:,:,2)+LE_1(:,:,2))
	avg_z   = (dot_product(zGrid, sum(LE_0(:,:,1) + LE_0(:,:,2), dim = 1)) &
				+dot_product(zGrid*(1.0d+0+H), sum(LE_1(:,:,1) + LE_1(:,:,2), dim = 1)))/sum(LE_0+LE_1)
	
	print *, 'Average prod: ', 'young:', avg_z_y, 'old: ', avg_z_o
	print *, 'all:', avg_z 
	
	
	!	tenure distribution
	
	print *, ' '
	print *, 'Tenure less than 2 years : ', 1.0d+2*sum(LE_0(1:7,:,:)+LE_1(1:7,:,:))/sum(LE_0+LE_1)
	print *, 'Tenure from 2 to 5 years : ', 1.0d+2*sum(LE_0(8:19,:,:)+LE_1(8:19,:,:))/sum(LE_0+LE_1)
	print *, 'Tenure from 5 to 10 years: ', 1.0d+2*sum(LE_0(20:39,:,:)+LE_1(20:39,:,:))/sum(LE_0+LE_1)
	print *, 'Tenure more than 10 years: ', 1.0d+2*sum(LE_0(40:NT,:,:)+LE_1(40:NT,:,:))/sum(LE_0+LE_1)
	
	
	!	separation rates

	!	young workers
	
	sep_y = 0.0d+0
	do it = 2, NT-1
	
		sep	= (1.0d+0-delta)*(1.0d+0-gamma)*matmul(transpose(P), (1.0d+0-pi_y(it-1,:))*LE_0(it-1,:,1))
		if (sepy_0(it) .le. NZ) sep_y = sep_y + sum(sep(sepy_0(it):NZ))
		sep = (1.0d+0-delta)*gamma*matmul(transpose(P), (1.0d+0-pi_y(it-1,:))*LE_0(it-1,:,1))
		if (sepo_0(it) .le. NZ) sep_y = sep_y + sum(sep(sepo_0(it):NZ))
		sep	= (1.0d+0-delta)*(1.0d+0-gamma)*matmul(transpose(P), pi_y(it-1,:)*LE_0(it-1,:,1)) &
				+ (1.0d+0-delta)*(1.0d+0-gamma)*matmul(transpose(P), LE_1(it-1,:,1))
		if (sepy_1(it) .le. NZ) sep_y = sep_y + sum(sep(sepy_1(it):NZ))
		sep = (1.0d+0-delta)*gamma*matmul(transpose(P), pi_y(it-1,:)*LE_0(it-1,:,1)) &
				+ (1.0d+0-delta)*gamma*matmul(transpose(P), LE_1(it-1,:,1))
		if (sepo_1(it) .le. NZ) sep_y = sep_y + sum(sep(sepo_1(it):NZ))
	
	end do
	sep = (1.0d+0-delta)*(1.0d+0-gamma)*matmul(transpose(P), (1.0d+0-pi_y(NT-1,:))*LE_0(NT-1,:,1) &
															  + (1.0d+0-pi_y(NT,:))*LE_0(NT,:,1))
	if (sepy_0(NT) .le. NZ) sep_y = sep_y + sum(sep(sepy_0(NT):NZ))
	sep = (1.0d+0-delta)*gamma*matmul(transpose(P), (1.0d+0-pi_y(NT-1,:))*LE_0(NT-1,:,1) + (1.0d+0-pi_y(NT,:))*LE_0(NT,:,1))
	if (sepo_0(NT) .le. NZ) sep_y = sep_y + sum(sep(sepo_0(NT):NZ))
	
	sep = (1.0d+0-delta)*(1.0d+0-gamma)*matmul(transpose(P), pi_y(NT-1,:)*LE_0(NT-1,:,1) + pi_y(NT,:)*LE_0(NT,:,1)) &
			+ (1.0d+0-delta)*(1.0d+0-gamma)*matmul(transpose(P), LE_1(NT-1,:,1) + LE_1(NT,:,1))
	if (sepy_1(NT) .le. NZ) sep_y = sep_y + sum(sep(sepy_1(NT):NZ))
	sep = (1.0d+0-delta)*gamma*matmul(transpose(P), pi_y(NT-1,:)*LE_0(NT-1,:,1) + pi_y(NT,:)*LE_0(NT,:,1)) &
			+ (1.0d+0-delta)*gamma*matmul(transpose(P), LE_1(NT-1,:,1) + LE_1(NT,:,1))
	if (sepo_1(NT) .le. NZ) sep_y = sep_y + sum(sep(sepo_1(NT):NZ))
	sep_y = sep_y + delta*sum(LE_0(:,:,1)+LE_1(:,:,1))
	
	!	old workers
	
	sep_o = 0.0d+0
	do it = 2, NT-1
		
		sep	= (1.0d+0-delta)*(1.0d+0-khi)*matmul(transpose(P), (1.0d+0-pi_o(it-1,:))*LE_0(it-1,:,2))
		if (sepo_0(it) .le. NZ) sep_o = sep_o + sum(sep(sepo_0(it):NZ))
		sep	= (1.0d+0-delta)*(1.0d+0-khi)*matmul(transpose(P), pi_o(it-1,:)*LE_0(it-1,:,2)) &
				+ (1.0d+0-delta)*(1.0d+0-khi)*matmul(transpose(P), LE_1(it-1,:,2))
		if (sepo_1(it) .le. NZ) sep_o = sep_o + sum(sep(sepo_1(it):NZ))
		
	end do
	sep = (1.0d+0-delta)*(1.0d+0-khi)*matmul(transpose(P), (1.0d+0-pi_o(NT-1,:))*LE_0(NT-1,:,2) + (1.0d+0-pi_o(NT,:))*LE_0(NT,:,2))
	if (sepo_0(NT) .le. NZ) sep_o = sep_o + sum(sep(sepo_0(NT):NZ))
	sep = (1.0d+0-delta)*(1.0d+0-khi)*matmul(transpose(P), pi_o(NT-1,:)*LE_0(NT-1,:,2) + pi_o(NT,:)*LE_0(NT,:,2)) &
			+ (1.0d+0-delta)*(1.0d+0-khi)*matmul(transpose(P), LE_1(NT-1,:,2) + LE_1(NT,:,2))
	if (sepo_1(NT) .le. NZ) sep_o = sep_o + sum(sep(sepo_1(NT):NZ))
	sep_o = sep_o + khi*sum(LE_0(:,:,2)+LE_1(:,:,2))
	sep_o = sep_o + (1.0d+0-khi)*delta*sum(LE_0(:,:,2)+LE_1(:,:,2))
	
	!	first 2 years
	
	sep_temp = 0.0d+0
	do it = 2, 9
		
		sep	= (1.0d+0-delta)*(1.0d+0-gamma)*matmul(transpose(P), (1.0d+0-pi_y(it-1,:))*LE_0(it-1,:,1))
		if (sepy_0(it) .le. NZ) sep_temp = sep_temp + sum(sep(sepy_0(it):NZ))		
		sep = (1.0d+0-delta)*(gamma*matmul(transpose(P), (1.0d+0-pi_y(it-1,:))*LE_0(it-1,:,1)) &
				 + (1.0d+0-khi)*matmul(transpose(P), (1.0d+0-pi_o(it-1,:))*LE_0(it-1,:,2)))
		if (sepo_0(it) .le. NZ) sep_temp = sep_temp + sum(sep(sepo_0(it):NZ))
		sep = (1.0d+0-delta)*(1.0d+0-gamma)*matmul(transpose(P), pi_y(it-1,:)*LE_0(it-1,:,1)) &
				+ (1.0d+0-delta)*(1.0d+0-gamma)*matmul(transpose(P), LE_1(it-1,:,1))
		if (sepy_1(it) .le. NZ) sep_temp = sep_temp + sum(sep(sepy_1(it):NZ))		
		sep = (1.0d+0-delta)*(gamma*matmul(transpose(P), pi_y(it-1,:)*LE_0(it-1,:,1)) &
					+ (1.0d+0-khi)*matmul(transpose(P), pi_o(it-1,:)*LE_0(it-1,:,2))) &
			+ (1.0d+0-delta)*(gamma*matmul(transpose(P), LE_1(it-1,:,1)) + (1.0d+0-khi)*matmul(transpose(P), LE_1(it-1,:,2)))
		if (sepo_1(it) .le. NZ) sep_temp = sep_temp + sum(sep(sepo_1(it):NZ))
		
	end do
	sep_temp = sep_temp + khi*sum(LE_0(1:8,:,2)+LE_1(1:8,:,2))
	sep_temp = sep_temp + delta*(sum(LE_0(1:8,:,1)+LE_1(1:8,:,1)) + (1.0d+0-khi)*sum(LE_0(1:8,:,2)+LE_1(1:8,:,2)))
	
	!	remaining separations
	
	sep_tot  = sep_y + sep_o
	sep_perm = sep_tot-sep_temp
	
	!	scale
	
	sep_y	 = sep_y/sum(LE_0(:,:,1)+LE_1(:,:,1))
	sep_o	 = sep_o/sum(LE_0(:,:,2)+LE_1(:,:,2))
	sep_temp = sep_temp/sum(LE_0(1:8,:,1)+LE_0(1:8,:,2)+LE_1(1:8,:,1)+LE_1(1:8,:,2))
	sep_perm = sep_perm/sum(LE_0(9:NT,:,1)+LE_0(9:NT,:,2)+LE_1(9:NT,:,1)+LE_1(9:NT,:,2))
	sep_tot  = sep_tot/sum(LE_0+LE_1)
	
	print *, ' '
	print *, 'Separation rate ', 'young:', 1.0d+2*sep_y, 'old: ', 1.0d+2*sep_o
	print *, 'Separation rate ', 'temp :', 1.0d+2*sep_temp, 'perm:', 1.0d+2*sep_perm
	print *, 'Separation rate (total)', 1.0d+2*sep_tot
	print *, 'Quit rate (total)', &
			  1.0d+2*delta*(sum(LE_0(:,:,1)+LE_1(:,:,1))+(1.0d+0-khi)*sum(LE_0(:,:,2)+LE_1(:,:,2)))/sum(LE_0+LE_1)
	print *, 'Quit rate (/total)', &
			  1.0d+2*delta*(sum(LE_0(:,:,1)+LE_1(:,:,1))+(1.0d+0-khi)*sum(LE_0(:,:,2)+LE_1(:,:,2)))/sum(LE_0+LE_1)/sep_tot
	print *, 'Job-finding rate', 1.0d+2*jfind
	print *, 'Vacancy cost    ', k
	print *, ' '
	
	!	nonemployment
	
	norm = sum(LU(:,1))+sum(LE_0(:,:,1)+LE_1(:,:,1))
	print *, 'Nonemployment rate ', 'young:', 1.0d+2*sum(LU(:,1))/norm, 'old: ', 1.0d+2*sum(LU(:,2:3))/(1.0d+0-norm)
	print *, 'Nonemployment rate ', 1.0d+2*sum(LU)
	print *, ' '
	
	print *, 'Equilibrium tightness ', theta
	print *, 'Equilibrium tax rate', 1.0d+2*kappa
	print *, ' '
	
	print *, 'Lifetime utility, newborn', Uy(1)
	print *, ' '
	print *, ' '
	
end subroutine


!	obtain kappa consistent with balanced budget constraint

double precision function kappa_budget_eq()

	kappa_budget_eq = (By*sum(LU(:,1))+Bo*sum(LU(:,2:3)))/(avg_w*sum(LE_0+LE_1))
	
end function


!	compute average wage in the economy

double precision function compute_avg_wage()

	double precision :: res
	
	res = 0.0d+0
	do it = 1, NT
	do iz = 1, NZ
		res = res + wwy_0(it,iz)*LE_0(it,iz,1) + wwo_0(it,iz)*LE_0(it,iz,2)
		res = res + wwy_1(it,iz)*LE_1(it,iz,1) + wwo_1(it,iz)*LE_1(it,iz,2)
	end do
	end do
	compute_avg_wage = res/sum(LE_0+LE_1)
	
end function


!	obtain theta consistent with free-entry

double precision function theta_free_entry()

	double precision :: res
	
	if (norm_theta) then
	
		k = A*rho*Jy_0(1,iz0)
		theta_free_entry = 1.0d+0
		
	else
	
		res = rho*Jy_0(1,iz0)
		res = k/res
		theta_free_entry = min(1.0d+0/res, (A/res)**(1.0d+0/psi))

	end if
		
end function

end module